/* db_types.h - type definitions for a simple virtual machine
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __DB_TYPES_H__
#define __DB_TYPES_H__

/**********/
/* Common */
/**********/

#define VMTRUE		1
#define VMFALSE   	0

/*********/
/* WIN32 */
/*********/

#ifdef WIN32

#include "../common/db_inttypes.h"

#include <stdio.h>
#include <string.h>

typedef FILE VMFILE;

#define VMCODE_SIZE		    (16 * 1024)
#define VMDATA_SIZE    	    11720
#define VMDATA_ATTR

#define FLASH_SPACE
#define FLASH_PAGESIZE      512
#define FLASH_BOOT

#define VMCODEBYTE(p)       *(uint8_t *)(p)
#define VMCOPY(d, s, c, h)  memcpy(d, s, c)
#define VMGETBYTE(p, h)     *(uint8_t *)(p)
#define VMSTRLEN(p)         strlen(p)

#define DEFAULT_HEAP_SIZE       1024
#define DEFAULT_HEAP_HANDLES    32

#endif  // WIN32

/**********/
/* XGSPIC */
/**********/

#ifdef XGSPIC

#include "../common/db_inttypes.h"

#include <stdio.h>
#include <string.h>
#include "libpic30.h"
#include "FSIO.h"

// not defined but actually located in the gcc library!
int vsnprintf(char *, size_t, const char *, va_list);

typedef FSFILE VMFILE;

#define VMCODE_SIZE		    (16 * 1024)
#define VMDATA_SIZE    	    11600
#define VMDATA_ATTR         __attribute__((far))

#define FLASH_SPACE         const
#define FLASH_PAGESIZE      (_FLASH_PAGE * 2)
#define FLASH_ROWSIZE       (_FLASH_ROW * 2)
#define FLASH_BOOT

#define VMCODEBYTE(p)       *(uint8_t *)(p)
#define VMCOPY(d, s, c, h)  memcpy(d, s, c)
#define VMGETBYTE(p, h)     *(uint8_t *)(p)
#define VMSTRLEN(p)         strlen(p)

#define DEFAULT_HEAP_SIZE       1024
#define DEFAULT_HEAP_HANDLES    32

#endif  // XGSPIC

/**********/
/* XGSAVR */
/**********/

#ifdef XGSAVR

#include <stdint.h>
#include <string.h>
#include <avr/boot.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

typedef void VMFILE;

#define VMCODE_SIZE		    (4 * 1024)
#define VMDATA_SIZE    	    2200
#define VMDATA_ATTR

#define FLASH_SPACE         PROGMEM
#define FLASH_PAGESIZE      SPM_PAGESIZE
#define FLASH_BOOT          BOOTLOADER_SECTION

#define VMCODEBYTE(p)	    pgm_read_byte(p)
#define VMCOPY(d, s, c, h)  do {                        \
                                if (h)                  \
                                    memcpy(d, s, c);    \
                                else                    \
                                    memcpy_P(d, s, c);  \
                            } while (0)
#define VMGETBYTE(p, h)     ((h) ? *(uint8_t *)(p) : VMCODEBYTE(p))
#define VMSTRLEN(p)         strlen_P(p)

#define DEFAULT_HEAP_SIZE       512
#define DEFAULT_HEAP_HANDLES    16

#endif  // XGSAVR

#endif
